part of tecfy_basic_package;

class ApiService {

  static String get token => ConfigService.token;

  static Future<OperationResult<Map>> httpGet(String url) async {
    try {
      var result = await httpGetString(url);
      if (!result.success)
        return OperationResult(success: false, message: result.message);
      var map = json.decode(result.data);
      print(map);
      return OperationResult(success: true, data: map);

    } catch (err) {
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<dynamic>> httpPost(String url,dynamic data,[bool asString=false]) async {
    try {

      HttpClient httpClient = new HttpClient();
      httpClient.connectionTimeout = Duration(seconds: 5);
      print('POST: ${_getFullUrl(url)}');
      print(data);
      HttpClientRequest request = await httpClient.postUrl(Uri.parse(_getFullUrl(url)));
      request.headers.set('content-type', 'application/json');
      if(token != null && token.isNotEmpty)
        request.headers.set('Authorization', 'Bearer $token');

      if(asString)
        data = json.encode(data).replaceAll('\"','\'');
      var body = json.encode(data);
      print(body);
      request.add(utf8.encode(body));
      HttpClientResponse response = await request.close();
      
      if(response.statusCode != 200 )//&& response.statusCode !=307
      {
        print(response.statusCode);
        return OperationResult(success: false, message: 'Result code = ${response.statusCode}');
      }

      String reply = await response.transform(utf8.decoder).join();
      
      httpClient.close();
      
      var r = jsonDecode(reply);
      print(r);
      return OperationResult(success: r['success'], message: r['errorString'],data: r['data'],errorCodeString: r['errorCodeString'] );
      // } else
      //   return OperationResult(success: false, message: 'Result code = ${result.statusCode}');
    } catch (err) {
      print('xxxxxxxxxxxx ERROR: POST');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }
  
  static Future<OperationResult<dynamic>> httpPut(String url,Map data) async {
    try {

      HttpClient httpClient = new HttpClient();
      httpClient.connectionTimeout = Duration(seconds: 5);
      print('PUT: ${_getFullUrl(url)}');
      print(data);
      HttpClientRequest request = await httpClient.putUrl(Uri.parse(_getFullUrl(url)));
      request.headers.set('content-type', 'application/json');
      if(token != null && token.isNotEmpty)
        request.headers.set('Authorization', 'Bearer $token');

      request.add(utf8.encode(json.encode(data)));
      HttpClientResponse response = await request.close();
      
      if(response.statusCode != 200 )//&& response.statusCode !=307
      {
        print(response.statusCode);
        return OperationResult(success: false, message: 'Result code = ${response.statusCode}');
      }

      String reply = await response.transform(utf8.decoder).join();
      
      httpClient.close();
      
      var r = jsonDecode(reply);
      print(r);
      return OperationResult(success: r['success'], message: r['errorString'],data: r['data'], errorCodeString: r['errorCodeString'], );
      // } else
      //   return OperationResult(success: false, message: 'Result code = ${result.statusCode}');
    } catch (err) {
      print('xxxxxxxxxxxx ERROR: POST');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<dynamic>> httpPostEx(String url,dynamic data) async {
    try {

      print('POST: ${_getFullUrl(url)}');
     
      var result = await http.post(_getFullUrl(url),
        body: json.encode(data),
        headers: {
          'content-type': 'application/json',
          'Authorization': 'Bearer $token'
        });
     
      if(result.statusCode != 200 )
      {
        print(result.statusCode);
        return OperationResult(success: false, message: 'Result code = ${result.statusCode}');
      }

      String reply =  result.body;
      
      var r = jsonDecode(reply);
      
      return OperationResult(success: r['success'], message: r['errorString'] ?? r['fullMessagesString'] ,data: r['data'], errorCodeString: r['errorCodeString'] );
      
    } catch (err) {
      print('xxxxxxxxxxxx ERROR: POST');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<dynamic>> httpPutEx(String url,dynamic data) async {
    try {

      print('PUT: ${_getFullUrl(url)}');
      print(data);

      var result = await http.put(_getFullUrl(url),
        body: json.encode(data),
        headers: {
          'content-type': 'application/json',
          'Authorization': 'Bearer $token'
        });
     
      if(result.statusCode != 200 )//&& response.statusCode !=307
      {
        print(result.statusCode);
        return OperationResult(success: false, message: 'Result code = ${result.statusCode}');
      }

      String reply =  result.body;
      
      var r = jsonDecode(reply);
      print(r);
      return OperationResult(success: r['success'], message: r['errorString'],data: r['data'], errorCodeString: r['errorCodeString'] );
      // } else
      //   return OperationResult(success: false, message: 'Result code = ${result.statusCode}');
    } catch (err) {
      print('xxxxxxxxxxxx ERROR: POST');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<dynamic>> httpGetDynamic(String url) async {
    try {
      
      print('GET --------------------- ${_getFullUrl(url)}');

      var response = await http.get(_getFullUrl(url),headers:{'content-type': 'application/json','Authorization': 'Bearer $token'} );

      if(response.statusCode != 200)return OperationResult(success: false, message: 'Result code = ${response.statusCode}');
      
      String reply = response.body;
      
      if(reply == null || reply.length ==0 )return OperationResult(success: false, message: 'Result is empty');
      
      var r = jsonDecode(reply);
      
      if(r['success'] != true) print(r);
      return OperationResult(success: r['success'], message: r['errorString'] ?? r['errorCodeString'] ?? r['fullMessagesString'] ?? 'No Server Error!',data: r['data'], );
     
    } catch (err) {
      print('xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx ERROR: GET: $url');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }
  
  static Future<OperationResult<dynamic>> httpDeleteDynamic(String url) async {
    try {      
      print('DELETE --------------------- ${_getFullUrl(url)}');
      
      var response = await http.delete(_getFullUrl(url),
        headers: {
          'content-type': 'application/json',
          'Authorization': 'Bearer $token'
        });

      
      if(response.statusCode != 200)return OperationResult(success: false, message: 'Result code = ${response.statusCode}');
      
      String reply = response.body;

      if(reply == null || reply.length ==0 )return OperationResult(success: false, message: 'Result is empty');
      
      var r = jsonDecode(reply);
      
      if(r['success'] != true)
        print(r);
      return OperationResult(success: r['success'], message: r['errorString'] ?? r['errorCodeString'] ?? r['fullMessagesString'] ?? 'No Server Error!',data: r['data'], );
     
    } catch (err) {
      print('xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   ERROR: DELETE: $url');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }
  
  static Future<OperationResult<dynamic>> httpDeleteDynamicEx(String url) async {
    try {      
      print('DELETE --------------------- ${_getFullUrl(url)}');

      HttpClient httpClient = new HttpClient();
      httpClient.connectionTimeout = Duration(seconds: 3);
      print('start create request');
      HttpClientRequest request = await httpClient.deleteUrl(Uri.parse(_getFullUrl(url)));
      request.headers.set('content-type', 'application/json');
      request.headers.set('Authorization', 'Bearer $token');
      print('start send command');
      HttpClientResponse response = await request.close();
      
      if(response.statusCode != 200)return OperationResult(success: false, message: 'Result code = ${response.statusCode}');
      
      String reply = await response.transform(utf8.decoder).join();
      httpClient.close(force: true);

      if(reply == null || reply.length ==0 )return OperationResult(success: false, message: 'Result is empty');
      
      var r = jsonDecode(reply);
      
     // try{
      //   print('******************************* Saving in cash for test porpuse   ::  ' + url);
      //   ConfigService.setMap(url,r);
      // } catch (err) {
      //   print('xxxxxxxxxxx-----------------xxxxxxxxxxxxxxx   Canot Set in cash .....: DELETE: $url');
      //   }
      if(r['success'] != true)
        print(r);
      return OperationResult(success: r['success'], message: r['errorString'] ?? r['errorCodeString'] ?? r['fullMessagesString'] ?? 'No Server Error!',data: r['data'], );
     
    } catch (err) {
      print('xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   ERROR: DELETE: $url');
      print(err);
      return OperationResult(success: false, message: err.toString());
    }
  }
  
  static Future<OperationResult<String>> httpGetString(String url) async {
    try {
      var result = await http.get(_getFullUrl(url));

      if (result.statusCode == 200) {
        return OperationResult(success: true, data: result.body);
      } else
        return OperationResult(
            success: false, message: 'Result code = ${result.statusCode}');
    } catch (err) {
      return OperationResult(success: false, message: err.toString());
    }
  }

  // static String apiURL = 'http://app.tecfy.co/api/';
  static String get apiURL => ConfigService.apiURL;
  static String _getFullUrl(String url) {
    if(url.startsWith('http'))return url;
    return "$apiURL$url";
  }

  static Future<OperationResult<int>> httpPostFile(String url, Uint8List fileData, String name, [bool isImage = true]) async{
    try{
        print('POST:---- ${_getFullUrl(url)}   $name');
        var request = new http.MultipartRequest("POST",Uri.parse( _getFullUrl(url)));
        request.headers['Authorization']= 'Bearer $token';
        // request.fields['user'] = 'someone@somewhere.com';
        request.files.add(http.MultipartFile.fromBytes(
            'file',
            fileData,
            filename: name,
            contentType: isImage ? MediaType("image",path.extension(name)) : MediaType('application', 'x-tar'),
        ));
        
        request.headers["Authorization"] = "Bearer $token";
        // request.headers["Accept"] = vimeoUploadHeader;
        request.headers["Content-Type"] = "image/jpg";
        request.headers["Content-Length"] = fileData.length.toString();

        var response = await request.send();
        
        if (response.statusCode == 200)
        {
          print("Uploaded!");
          var data = (await response.stream.first);
          // print(utf8.decode(data));
          var r = json.decode(utf8.decode(data));
          return OperationResult(success: r['success'], message: r['errorString'],data: r['data'], errorCodeString: r['errorCodeString'] );
        }
      return OperationResult(success: false, message: 'Error: ${response.statusCode}');
    }catch(err){
      return OperationResult(success: false, message: 'Error!!');
    }
  }
}
